@@ -9,7 +9,6 @@
 # include <swap.h>
 #endif
 
-
 /*
 typedef struct {
          word32 S[4][256],P[18];
@@ -71,7 +70,7 @@ word32 F(blf_ctx * bc, word32 x)
 }
 
 /* x should be a 64 bit integer */
-void enblf(blf_ctx *c,word32 *x)
+void _mcrypt_enblf(blf_ctx *c,word32 *x)
 {
   word32 Xl;
   word32 Xr, temp;
@@ -141,7 +140,7 @@ void enblf_noswap(blf_ctx *c,word32 *x) /* Used internally */
  x[1] = Xr;
 }
 
-void deblf(blf_ctx *c,word32 *x)
+void _mcrypt_deblf(blf_ctx *c,word32 *x)
 {
   word32 Xl,Xr,temp;
   short i;
@@ -421,6 +420,6 @@ word32 pi[] = {
 return 0;
 }
 
-void blf_key(blf_ctx *c,char *k,int len) {
+void _mcrypt_blf_key(blf_ctx *c,char *k,int len) {
      initialize_blowfish(c,k,len);
 }
