@@ -1,4 +1,5 @@
-/* Copyright (C) 1998,1999 Nikos Mavroyanopoulos This library is free
+/* Copyright (C) 1998,1999 Nikos Mavroyanopoulos
+ * This library is free
  * software; you can redistribute it and/or modify it under the terms of the
  * GNU Library General Public License as published by the Free Software
  * Foundation; either version 2 of the License, or (at your option) any
@@ -27,61 +28,67 @@
 #include <gost.h>
 #include <des.h>
 #include <safer.h>
-#include <cast.h>
+#include <cast-128.h>
+#include <cast-256.h>
 #include <tean.h>
 #include <aes.h>
 #include <xmemory.h>
 #include <rc2.h>
 #include <rc6.h>
 #include <idea.h>
-#include <sha1.h>		/* for gen_key */
-
-#define MAX_THREADS 256		/* After that pointers are being reused */
+#include <saferplus.h>
+#include <loki97.h>
 
 /* I think now is re-entrant */
 
-static int check_pointer[MAX_THREADS] =
-{0};
+static int init_check_pointer=0;
+static int check_pointer[MAX_THREADS];
 static int used_all_threads = 0;
 
 #ifdef HAVE_LIBPTHREAD
-#include <pthread.h>
-static int cur_thread = 0;
-pthread_mutex_t counter_cur_thread = PTHREAD_MUTEX_INITIALIZER;
+# include <pthread.h>
+ static int cur_thread = 0;
+ pthread_mutex_t counter_cur_thread = PTHREAD_MUTEX_INITIALIZER;
+# define LOCK pthread_mutex_lock(&counter_cur_thread)
+# define UNLOCK pthread_mutex_unlock(&counter_cur_thread)
 #else
-static int cur_thread = 0;	/* No threads */
+# define LOCK 
+# define UNLOCK 
+ static int cur_thread = 0;	/* No threads */
 #endif
 
-static struct cast_key *ckey = NULL;	/* For cast */
-static blf_ctx *c = NULL;	/* For Blowfish */
-
-static cipherInstance *CI = NULL;	/* twofish */
-static keyInstance *KI = NULL;
 
+static char **ckey = NULL;
+static char **c = NULL;     /* For Blowfish */
+static char **CI = NULL;     /* twofish */
+static char **KI = NULL;
+static char **saferplus_key=NULL;
+static char **loki97_key=NULL;
 
 static char **kn1 = NULL;	/* For DES */
 static char **kn2 = NULL;
 static char **kn3 = NULL;	/* For tripleDES */
 static word8 **saferkey = NULL;	/* For safer64 and 128 */
-static word16 **rc2_key;	/* For rc2 */
+static word8 **rc2_key=NULL;	/* For rc2 */
+static char **cast256_key=NULL; /* cast-256 */
 
 #ifdef NON_FREE
-static word16 **idea_ikey;
-static word16 **idea_key;	/* For idea */
+static char **idea_ikey;
+static char **idea_key;	/* For idea */
 
-static word32 **rc6_skey;	/* For RC6 */
+static char **rc6_skey;	/* For RC6 */
 #endif
 
 /* For CBC mode */
-static word32 **previous_ciphertext;
-static word32 **previous_plaintext;
-static word32 **previous_cipher;
+static word32 **previous_ciphertext=NULL;
+static word32 **previous_plaintext=NULL;
+static word32 **previous_cipher=NULL;
 
 /* For CFB mode */
-static word8 **sd_register;
-static word8 **s_register;
-static word8 **enc_s_register;
-static word8 **enc_sd_register;
+static word8 **sd_register=NULL;
+static word8 **s_register=NULL;
+static word8 **enc_s_register=NULL;
+static word8 **enc_sd_register=NULL;
 
 
 
@@ -99,30 +106,38 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key, int
 	char twokey[68];
 	int i, j;
 
+        if (!init_check_pointer) {
+		init_check_pointer=1;
+        	for (i=0;i<sizeof(check_pointer);i++) {
+        	       check_pointer[i]=0;
+        	}
+        }
+
 	if (lenofkey == 0)
 		return -1;	/* error */
 
-
 	if (used_all_threads == 0) {
 		saferkey = mxrealloc(saferkey, (thread + 1) * sizeof(char *));
 		kn1 = mxrealloc(kn1, (thread + 1) * sizeof(char *));
 		kn2 = mxrealloc(kn2, (thread + 1) * sizeof(char *));
 		kn3 = mxrealloc(kn3, (thread + 1) * sizeof(char *));
-		c = mxrealloc(c, (thread + 1) * sizeof(blf_ctx));
-		CI = mxrealloc(CI, (thread + 1) * sizeof(cipherInstance));
-		KI = mxrealloc(KI, (thread + 1) * sizeof(keyInstance));
-		ckey = mxrealloc(ckey, (thread + 1) * sizeof(struct cast_key));
-		keyword_given = mxrealloc(keyword_given, (thread + 1) * sizeof(char *));
-		rc2_key = mxrealloc(rc2_key, (thread + 1) * sizeof(word16 *));
-
+		c = mxrealloc(c, (thread + 1) * sizeof(char*));
+		CI = mxrealloc(CI, (thread + 1) * sizeof(char*));
+		KI = mxrealloc(KI, (thread + 1) * sizeof(char*));
+		ckey = mxrealloc(ckey, (thread + 1) * sizeof(word8*));		
+		keyword_given = mxrealloc(keyword_given, (thread + 1) * sizeof(word8 *));
+		rc2_key = mxrealloc(rc2_key, (thread + 1) * sizeof(char*));
+		cast256_key = mxrealloc( cast256_key, (thread + 1) * sizeof(char*));
+		saferplus_key = mxrealloc( saferplus_key, (thread + 1) * sizeof(char*));
+		loki97_key = mxrealloc (loki97_key, (thread + 1) * sizeof(char*));
 #ifdef NON_FREE
-		rc6_skey = mxrealloc(rc6_skey, (thread + 1) * sizeof(word32 *));
-		idea_key = mxrealloc(idea_key, (thread + 1) * sizeof(word16 *));
-		idea_ikey = mxrealloc(idea_ikey, (thread + 1) * sizeof(word16 *));
+		rc6_skey = mxrealloc(rc6_skey, (thread + 1) * sizeof(char*));
+		idea_key = mxrealloc(idea_key, (thread + 1) * sizeof(char*));
+		idea_ikey = mxrealloc(idea_ikey, (thread + 1) * sizeof(char*));
 #endif
 
-		algorithm_given = mxrealloc(algorithm_given, (thread + 1) * sizeof(int));
-		mode_given = mxrealloc(mode_given, (thread + 1) * sizeof(int));
+		algorithm_given = realloc(algorithm_given, (thread + 1) * sizeof(int));
+		mode_given = realloc(mode_given, (thread + 1) * sizeof(int));
 
 	}
 	if (mcrypt_get_key_size(algorithm) < lenofkey)
@@ -132,16 +147,18 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key, int
 	algorithm_given[thread] = algorithm;
 	mode_given[thread] = mode;
 	keyword_given[thread] = mxcalloc(1, mcrypt_get_key_size(algorithm));
+
 	memmove(keyword_given[thread], key, lenofkey);
+
 	safer_skey = keyword_given[thread];
 
 
 	switch (algorithm) {
 
 	case DES:
-		desinit(0);
+		_mcrypt_desinit(0);
 		kn1[thread] = mxmalloc(8 * 16 * sizeof(char));
-		des_setkey((void *) kn1[thread], (void *) keyword_given[thread]);
+		_mcrypt_des_setkey((void *) kn1[thread], (void *) keyword_given[thread]);
 		return thread;
 		break;
 
@@ -150,19 +167,32 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key, int
 		memmove(keyword1, keyword_given[thread], 8);
 		saferkey[thread] = mxmalloc(1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
 
-		Safer_Init_Module();
-		Safer_Expand_Userkey((unsigned char *) keyword1, (unsigned char *) keyword1, SAFER_SK64_DEFAULT_NOF_ROUNDS, 1, saferkey[thread]);
+		_mcrypt_Safer_Init_Module();
+		_mcrypt_Safer_Expand_Userkey((unsigned char *) keyword1, (unsigned char *) keyword1, SAFER_SK64_DEFAULT_NOF_ROUNDS, 1, saferkey[thread]);
 		/* 8 rounds and secure key schedule */
-		free(keyword1);
+		mxfree(keyword1, 8);
 
 	case CAST128:
-		cast_setkey(&ckey[thread], (void *) keyword_given[thread], lenofkey);
+                ckey[thread] = mxmalloc( sizeof(struct cast_key));
+		_mcrypt_cast_setkey( (void*) ckey[thread], (void *) keyword_given[thread], lenofkey);
+		return thread;
+		break;
+
+	case LOKI97:
+		loki97_key[thread] = mxmalloc(96*sizeof(word32));
+		_mcrypt_loki97_set_key( (void*)loki97_key[thread], (void*) keyword_given[thread], lenofkey);
+		return thread;
+		break;
+		
+	case CAST256:
+                cast256_key[thread] = mxmalloc(96*sizeof(word32));
+                _mcrypt_cast256_set_key( (void*)cast256_key[thread], (void *) keyword_given[thread], lenofkey);
 		return thread;
 		break;
 
 	case RC2:
-		rc2_key[thread] = mxmalloc(64 * sizeof(word16));
-		rc2_keyschedule(rc2_key[thread], (void *) keyword_given[thread], lenofkey, 1024);
+		rc2_key[thread] = mxcalloc(1, 64 * sizeof(word16));
+		_mcrypt_rc2_keyschedule( (void*)rc2_key[thread], (void *) keyword_given[thread], lenofkey);
 		return thread;
 		break;
 
@@ -170,39 +200,41 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key, int
 	case IDEA:
 		idea_key[thread] = mxmalloc(52 * sizeof(word16));
 		idea_ikey[thread] = mxmalloc(52 * sizeof(word16));
-		Idea_ExpandUserKey((void *) keyword_given[thread], idea_key[thread]);
-		Idea_InvertKey(idea_key[thread], idea_ikey[thread]);
+		_mcrypt_Idea_ExpandUserKey((void *) keyword_given[thread], (void*)idea_key[thread]);
+		_mcrypt_Idea_InvertKey( (void*) idea_key[thread], (void*) idea_ikey[thread]);
 		return thread;
 		break;
 
 	case RC6:
-		rc6_skey[thread] = mxmalloc(RC6_FACTOR * sizeof(word32));
-		rc6_schedule((void *) keyword_given[thread], rc6_skey[thread]);
+		rc6_skey[thread] = mxmalloc(44*sizeof(word32));
+		_mcrypt_rc6_set_key( (void*)rc6_skey[thread], (void *) keyword_given[thread], lenofkey);
+
 		return thread;
 		break;
 #endif
 
 	case SAFER128:
-		Safer_Init_Module();
+		_mcrypt_Safer_Init_Module();
 		keyword1 = mxmalloc(8);
 		keyword2 = mxmalloc(8);
 		saferkey[thread] = mxmalloc(1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
 
 		memmove(keyword1, safer_skey, 8);
 		memmove(keyword2, &safer_skey[8], 8);
-		Safer_Expand_Userkey((unsigned char *) keyword1, (unsigned char *) keyword2, SAFER_SK128_DEFAULT_NOF_ROUNDS, 1, saferkey[thread]);
+		_mcrypt_Safer_Expand_Userkey((unsigned char *) keyword1, (unsigned char *) keyword2, SAFER_SK128_DEFAULT_NOF_ROUNDS, 1, saferkey[thread]);
 		/* 8 rounds and secure key schedule */
-		free(keyword1);
-		free(keyword2);
+		mxfree(keyword1, 8);
+		mxfree(keyword2, 8);
 
 		return thread;
 		break;
 
 	case TripleDES:
-		desinit(0);
+		_mcrypt_desinit(0);
 		keyword1 = mxcalloc(1, 8 * sizeof(char));
 		keyword2 = mxcalloc(1, 8 * sizeof(char));
 		keyword3 = mxcalloc(1, 8 * sizeof(char));
+
 		kn1[thread] = mxmalloc(8 * 16 * sizeof(char));
 		kn2[thread] = mxmalloc(8 * 16 * sizeof(char));
 		kn3[thread] = mxmalloc(8 * 16 * sizeof(char));
@@ -210,27 +242,37 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key, int
 		/* If it returns 1 or -1 the key is too small */
 
 		if (BreakToThree(keyword_given[thread], lenofkey, keyword1, keyword2, keyword3) != 0) {
-			free(keyword1);
-			free(keyword2);
-			free(keyword3);
+			mxfree(keyword1, 8);
+			mxfree(keyword2, 8);
+			mxfree(keyword3, 8);
 			end_mcrypt(thread);
 			return (-1);
 		}
-		des_setkey((void *) kn1[thread], keyword1);
-		des_setkey((void *) kn2[thread], keyword2);
-		des_setkey((void *) kn3[thread], keyword3);
-		free(keyword1);
-		free(keyword2);
-		free(keyword3);
+		_mcrypt_des_setkey((void *) kn1[thread], keyword1);
+		_mcrypt_des_setkey((void *) kn2[thread], keyword2);
+		_mcrypt_des_setkey((void *) kn3[thread], keyword3);
+		mxfree(keyword1, 8);
+		mxfree(keyword2, 8);
+		mxfree(keyword3, 8);
 
 		return thread;
 		break;
 
-	case BLOWFISH:
+	case SAFERPLUS:
+		saferplus_key[thread] = mxmalloc( sizeof(SPI));
+		_mcrypt_saferplus_set_key( (void*)saferplus_key[thread], (void *) keyword_given[thread], lenofkey);
+		return thread;
+		break;
+		
+	case BLOWFISH448:
+	case BLOWFISH128:
+	case BLOWFISH192:
+	case BLOWFISH256:
+                c[cur_thread] = mxmalloc( sizeof(blf_ctx));
 		if (lenofkey < 2) {
-			blf_key(&c[thread], (void *) keyword_given[thread], 2);
+			_mcrypt_blf_key((void*)c[thread], (void *) keyword_given[thread], 2);
 		} else {
-			blf_key(&c[thread], (void *) keyword_given[thread], lenofkey);
+			_mcrypt_blf_key((void*)c[thread], (void *) keyword_given[thread], lenofkey);
 		}
 		return thread;
 		break;
@@ -239,9 +281,10 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key, int
 	case TWOFISH192:
 	case TWOFISH256:
 
-		Bzero(&KI[cur_thread], sizeof(keyInstance));
-		Bzero(&CI[cur_thread], sizeof(cipherInstance));
-		cipherInit(&CI[cur_thread], MODE_ECB, NULL);
+                CI[cur_thread] = mxcalloc(1, sizeof(cipherInstance));
+                KI[cur_thread] = mxcalloc(1, sizeof(keyInstance));
+
+		_mcrypt_cipherInit((void*)CI[cur_thread], MODE_ECB, NULL);
 
 		j = 0;
 		for (i = 0; i < lenofkey; i++) {
@@ -253,7 +296,7 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key, int
 			j += 2;
 		}
 
-		j = makeKey(&KI[cur_thread], DIR_ENCRYPT, mcrypt_get_key_size(algorithm_given[cur_thread]) * 8, twokey);
+		j = _mcrypt_makeKey((void*)KI[cur_thread], DIR_ENCRYPT, mcrypt_get_key_size(algorithm_given[cur_thread]) * 8, twokey);
 		if (j < 0) {
 			return (-1);
 		}
@@ -261,7 +304,7 @@ int init_mcrypt(const int mode, const int algorithm, int thread, void *key, int
 		break;
 
 	case GOST:
-		kboxinit();
+		_mcrypt_kboxinit();
 		return thread;
 		break;
 
@@ -289,68 +332,86 @@ int mcrypt(int thread, void *plaintext)
 
 	case SAFER64:
 	case SAFER128:
-		Safer_Encrypt_Block(plaintext, saferkey[thread]);
+		_mcrypt_Safer_Encrypt_Block(plaintext, saferkey[thread]);
 		return 0;
 		break;
 
+	case SAFERPLUS:
+		_mcrypt_saferplus_encrypt( (void*)saferplus_key[thread], plaintext);
+		return 0;
+		break;
+		
 	case CAST128:
-		cast_encrypt(&ckey[thread], (void *) plaintext);
+		_mcrypt_cast_encrypt((void*)ckey[thread], (void *) plaintext);
+		return 0;
+		break;
+
+	case LOKI97:
+		_mcrypt_loki97_encrypt((void*)loki97_key[thread], (void *) plaintext);
+		return 0;
+		break;
+		
+	case CAST256:
+		_mcrypt_cast256_encrypt((void*)cast256_key[thread], (void *) plaintext);
 		return 0;
 		break;
 
 	case RC2:
-		rc2_encrypt(rc2_key[thread], (void *) plaintext, (void *) plaintext);
+		_mcrypt_rc2_encrypt( (void*)rc2_key[thread], (void *) plaintext);
 		return 0;
 		break;
 
 #ifdef NON_FREE
 	case IDEA:
-		Idea_Crypt((void *) plaintext, (void *) plaintext, idea_key[thread]);
+		_mcrypt_Idea_Crypt((void *) plaintext, (void *) plaintext, (void*)idea_key[thread]);
 		return 0;
 		break;
 
 	case RC6:
-		rc6_encrypt((void *) plaintext, rc6_skey[thread]);
+		_mcrypt_rc6_encrypt( (void*)rc6_skey[thread], (void *) plaintext);
 		return 0;
 		break;
 #endif
 
 	case DES:
-		endes((void *) kn1[thread], plaintext);
+		_mcrypt_endes((void *) kn1[thread], plaintext);
 		return 0;
 		break;
 
 	case TripleDES:
-		endes((void *) kn1[thread], plaintext);
-		dedes((void *) kn2[thread], plaintext);
-		endes((void *) kn3[thread], plaintext);
+		_mcrypt_endes((void *) kn1[thread], plaintext);
+		_mcrypt_dedes((void *) kn2[thread], plaintext);
+		_mcrypt_endes((void *) kn3[thread], plaintext);
 		return 0;
 		break;
 
-	case BLOWFISH:
-		enblf(&c[thread], plaintext);
+	case BLOWFISH448:
+	case BLOWFISH128:
+	case BLOWFISH192:
+	case BLOWFISH256:
+		_mcrypt_enblf((void*)c[thread], plaintext);
 		return 0;
 		break;
 
 	case TWOFISH128:
 	case TWOFISH192:
 	case TWOFISH256:
-		blockEncrypt(&CI[thread], &KI[thread], plaintext, 16 * 8, plaintext);
+		_mcrypt_blockEncrypt((void*)CI[thread], (void*)KI[thread], plaintext, 16 * 8, plaintext);
 		return 0;
 		break;
 
 	case GOST:
-		gostcrypt(plaintext, (void *) keyword_given[thread]);
+		_mcrypt_gostcrypt(plaintext, (void *) keyword_given[thread]);
 		return 0;
 		break;
 
 	case TEAN:
-		cl_enc_block((void *) keyword_given[thread], plaintext);
+		_mcrypt_cl_enc_block((void *) keyword_given[thread], plaintext);
 		return 0;
 		break;
 
 	case ThreeWAY:
-		en3way(plaintext, (void *) keyword_given[thread]);
+		_mcrypt_en3way(plaintext, (void *) keyword_given[thread]);
 		return 0;
 		break;
 
@@ -368,68 +429,87 @@ int mdecrypt(int thread, void *plaintext)
 
 	case SAFER64:
 	case SAFER128:
-		Safer_Decrypt_Block(plaintext, (void *) saferkey[thread]);
+		_mcrypt_Safer_Decrypt_Block(plaintext, (void *) saferkey[thread]);
 		return 0;
 		break;
 
+	case SAFERPLUS:
+		_mcrypt_saferplus_decrypt( (void*)saferplus_key[thread], plaintext);
+		return 0;
+		break;
+
+
 	case CAST128:
-		cast_decrypt(&ckey[thread], (void *) plaintext);
+		_mcrypt_cast_decrypt((void*)ckey[thread], (void *) plaintext);
+		return 0;
+		break;
+
+	case LOKI97:
+		_mcrypt_loki97_decrypt((void*)loki97_key[thread], (void *) plaintext);
+		return 0;
+		break;
+
+	case CAST256:
+		_mcrypt_cast256_decrypt((void*)cast256_key[thread], (void *) plaintext);
 		return 0;
 		break;
 
 	case RC2:
-		rc2_decrypt(rc2_key[thread], (void *) plaintext, (void *) plaintext);
+		_mcrypt_rc2_decrypt( (void*) rc2_key[thread], (void *) plaintext);
 		return 0;
 		break;
 
 #ifdef NON_FREE
 	case IDEA:
-		Idea_Crypt((void *) plaintext, (void *) plaintext, idea_ikey[thread]);
+		_mcrypt_Idea_Crypt((void *) plaintext, (void *) plaintext, (void*) idea_ikey[thread]);
 		return 0;
 		break;
 
 	case RC6:
-		rc6_decrypt((void *) plaintext, rc6_skey[thread]);
+		_mcrypt_rc6_decrypt( (void*) rc6_skey[thread], (void *) plaintext);
 		return 0;
 		break;
 #endif
 
 	case DES:
-		dedes((char (*)[8]) kn1[thread], plaintext);
+		_mcrypt_dedes((char (*)[8]) kn1[thread], plaintext);
 		return 0;
 		break;
 
 	case TEAN:
-		cl_dec_block((void *) keyword_given[thread], plaintext);
+		_mcrypt_cl_dec_block((void *) keyword_given[thread], plaintext);
 		return 0;
 		break;
 
 	case TripleDES:
-		dedes((void *) kn3[thread], plaintext);
-		endes((void *) kn2[thread], plaintext);
-		dedes((void *) kn1[thread], plaintext);
+		_mcrypt_dedes((void *) kn3[thread], plaintext);
+		_mcrypt_endes((void *) kn2[thread], plaintext);
+		_mcrypt_dedes((void *) kn1[thread], plaintext);
 		return 0;
 		break;
 
 	case TWOFISH128:
 	case TWOFISH192:
 	case TWOFISH256:
-		blockDecrypt(&CI[thread], &KI[thread], plaintext, 16 * 8, plaintext);
+		_mcrypt_blockDecrypt((void*)CI[thread], (void*)KI[thread], plaintext, 16 * 8, plaintext);
 		return 0;
 		break;
 
-	case BLOWFISH:
-		deblf(&c[thread], plaintext);
+	case BLOWFISH448:
+	case BLOWFISH128:
+	case BLOWFISH192:
+	case BLOWFISH256:
+		_mcrypt_deblf((void*)c[thread], plaintext);
 		return 0;
 		break;
 
 	case GOST:
-		gostdecrypt(plaintext, (void *) keyword_given[thread]);
+		_mcrypt_gostdecrypt(plaintext, (void *) keyword_given[thread]);
 		return 0;
 		break;
 
 	case ThreeWAY:
-		de3way(plaintext, (void *) keyword_given[thread]);
+		_mcrypt_de3way(plaintext, (void *) keyword_given[thread]);
 		return 0;
 		break;
 
@@ -442,71 +522,72 @@ int mdecrypt(int thread, void *plaintext)
 int end_mcrypt(int td)
 {
 
-	Bzero(keyword_given[td], mcrypt_get_key_size(algorithm_given[td]));
-	free(keyword_given[td]);
+	mxfree(keyword_given[td], mcrypt_get_key_size(algorithm_given[td]));
 	check_pointer[td] = 0;
 
 	switch (algorithm_given[td]) {
 
 	case DES:
-		desdone();
-		Bzero(kn1[td], 8 * 16 * sizeof(char));
-		free(kn1[td]);
+		_mcrypt_desdone();
+		mxfree(kn1[td], 8 * 16 * sizeof(char));
 		break;
 
 	case TripleDES:
-		desdone();
-		Bzero(kn1[td], 8 * 16 * sizeof(char));
-		Bzero(kn2[td], 8 * 16 * sizeof(char));
-		Bzero(kn3[td], 8 * 16 * sizeof(char));
-		free(kn1[td]);
-		free(kn2[td]);
-		free(kn3[td]);
+		_mcrypt_desdone();
+		mxfree(kn1[td], 8 * 16 * sizeof(char));
+		mxfree(kn2[td], 8 * 16 * sizeof(char));
+		mxfree(kn3[td], 8 * 16 * sizeof(char));
 		break;
 
 	case RC2:
-		Bzero(rc2_key[td], 64 * sizeof(word16));
-		free(rc2_key[td]);
+		mxfree(rc2_key[td], 64 * sizeof(word16));
 		break;
 
 #ifdef NON_FREE
 	case IDEA:
-		Bzero(idea_key[td], 52 * sizeof(word16));
-		Bzero(idea_key[td], 52 * sizeof(word16));
-		free(idea_key[td]);
-		free(idea_ikey[td]);
+		mxfree(idea_key[td], 52 * sizeof(word16));
+		mxfree(idea_ikey[td], 52 * sizeof(word16));
 		break;
 
 	case RC6:
-		Bzero(rc6_skey[td], RC6_FACTOR * sizeof(word32));
-		free(rc6_skey[td]);
+		mxfree(rc6_skey[td], 44*sizeof(word32));
 		break;
 #endif
 
 	case TWOFISH128:
 	case TWOFISH192:
 	case TWOFISH256:
-		Bzero(&CI[td], sizeof(cipherInstance));
-		Bzero(&KI[td], sizeof(keyInstance));
+		mxfree(CI[td], sizeof(cipherInstance));
+		mxfree(KI[td], sizeof(keyInstance));
 		break;
 
-	case BLOWFISH:
-		Bzero(&c[td], sizeof(blf_ctx));
+	case BLOWFISH448:
+	case BLOWFISH128:
+	case BLOWFISH192:
+	case BLOWFISH256:
+		mxfree(c[td], sizeof(blf_ctx));
 		break;
 
 	case TEAN:
-		break;
-
 	case GOST:
 	case ThreeWAY:
+	        break;
+	        
 	case CAST128:
-		Bzero(&ckey[td], sizeof(struct cast_key));
+		mxfree(ckey[td], sizeof(struct cast_key));
+		break;
+
+	case LOKI97:
+		mxfree(loki97_key[td], 96*sizeof(word32));
+		break;
+		                
+	case CAST256:
+		mxfree(cast256_key[td], 96*sizeof(word32));
 		break;
 
 	case SAFER64:
 	case SAFER128:
-		Bzero(saferkey[td], 1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
-		free(saferkey[td]);
+		mxfree(saferkey[td], 1 + SAFER_BLOCK_LEN * (1 + 2 * SAFER_MAX_NOF_ROUNDS));
 		break;
 		break;
 
@@ -523,10 +604,8 @@ int init_mcrypt_ecb(const int algorithm, void *key, int lenofkey)
 {
 	int x;
 
-#ifdef HAVE_LIBPTHREAD
-	pthread_mutex_lock(&counter_cur_thread);
-#endif
-
+	LOCK;
+	
 	if (cur_thread == MAX_THREADS) {
 		cur_thread = 0;
 		used_all_threads = 1;
@@ -536,23 +615,17 @@ int init_mcrypt_ecb(const int algorithm, void *key, int lenofkey)
 	}
 
 	if (cur_thread >= MAX_THREADS) {
-#ifdef HAVE_LIBPTHREAD
-		pthread_mutex_unlock(&counter_cur_thread);
-#endif
+		UNLOCK;
 		return -1;
 	}
 	x = init_mcrypt(ECB, algorithm, cur_thread, key, lenofkey);
 
 	if (x < 0) {
 		end_mcrypt(cur_thread);
-#ifdef HAVE_LIBPTHREAD
-		pthread_mutex_unlock(&counter_cur_thread);
-#endif
+		UNLOCK;
 		return -1;
 	} else {
-#ifdef HAVE_LIBPTHREAD
-		pthread_mutex_unlock(&counter_cur_thread);
-#endif
+		UNLOCK;
 		return x;
 	}
 
@@ -598,9 +671,9 @@ int mdecrypt_ecb(int td, void *ciphertext, int len)
 int end_mcrypt_cbc(int td)
 {
 
-	free(previous_ciphertext[td]);
-	free(previous_cipher[td]);
-	free(previous_plaintext[td]);
+	mxfree(previous_ciphertext[td], mcrypt_get_block_size(algorithm_given[td]));
+	mxfree(previous_cipher[td], mcrypt_get_block_size(algorithm_given[td]));
+	mxfree(previous_plaintext[td], mcrypt_get_block_size(algorithm_given[td]));
 
 	return end_mcrypt(td);
 
@@ -612,61 +685,100 @@ int init_mcrypt_cbc(const int algorithm, void *key, int lenofkey)
 
 	int x;
 
-#ifdef HAVE_LIBPTHREAD
-	pthread_mutex_lock(&counter_cur_thread);
-#endif
-	cur_thread++;
 
-	if (cur_thread < MAX_THREADS && check_pointer[cur_thread] == 0) {
+	LOCK;
 
-/* for cbc */
-		if (used_all_threads == 0) {
-			previous_ciphertext = mxrealloc(previous_ciphertext, (cur_thread + 1) * sizeof(word32 *));
-			previous_cipher = mxrealloc(previous_cipher, (cur_thread + 1) * sizeof(word32 *));
-			previous_plaintext = mxrealloc(previous_plaintext, (cur_thread + 1) * sizeof(word32 *));
-		}
-/* end cbc */
-
-	} else {
+	if (cur_thread == MAX_THREADS) {
 		cur_thread = 0;
 		used_all_threads = 1;
-		while (check_pointer[cur_thread] == 1 && cur_thread <= MAX_THREADS) {
-			cur_thread++;
-		}
+	}
+	while (check_pointer[cur_thread] == 1 && cur_thread < MAX_THREADS) {
+		cur_thread++;
+	}
 
-		if (cur_thread >= MAX_THREADS) {
-#ifdef HAVE_LIBPTHREAD
-			pthread_mutex_unlock(&counter_cur_thread);
-#endif
-			return (-1);
+	if (cur_thread >= MAX_THREADS) {
+		UNLOCK;
+		return -1;
+	} else {
+                if (used_all_threads==0) {
+			previous_ciphertext = mxrealloc(previous_ciphertext, (cur_thread + 1) * sizeof(char*));
+			previous_cipher = mxrealloc(previous_cipher, (cur_thread + 1) * sizeof(char*));
+			previous_plaintext = mxrealloc(previous_plaintext, (cur_thread + 1) * sizeof(char*));
 		}
 	}
 
 
+
 /* For cbc */
 	previous_ciphertext[cur_thread] = mxcalloc(1, mcrypt_get_block_size(algorithm));
 	previous_cipher[cur_thread] = mxmalloc(mcrypt_get_block_size(algorithm));
 	previous_plaintext[cur_thread] = mxcalloc(1, mcrypt_get_block_size(algorithm));
-
 /* End cbc */
 
 	x = init_mcrypt(CBC, algorithm, cur_thread, key, lenofkey);
 
 	if (x < 0) {
 		end_mcrypt_cbc(cur_thread);
-#ifdef HAVE_LIBPTHREAD
-		pthread_mutex_unlock(&counter_cur_thread);
-#endif
+		UNLOCK;
 		return -1;
 	} else {
-#ifdef HAVE_LIBPTHREAD
-		pthread_mutex_unlock(&counter_cur_thread);
-#endif
+		UNLOCK;
 		return x;
 	}
 
 }
 
+int init_mcrypt_cbc_iv(const int algorithm, void *key, int lenofkey, void *IV)
+{
+
+	int x, tmpi=MAX_THREADS;
+
+	LOCK;
+
+	if (cur_thread >= MAX_THREADS) {
+		cur_thread = 0;
+		used_all_threads = 1;
+	}
+	while (check_pointer[cur_thread] == 1 && cur_thread < MAX_THREADS) {
+		cur_thread++;
+		tmpi--;
+		if (tmpi<0) break;
+	}
+
+	if (cur_thread >= MAX_THREADS) {
+		UNLOCK;
+		return -1;
+	} else {
+                if (used_all_threads==0) {
+			previous_ciphertext = mxrealloc(previous_ciphertext, (cur_thread + 1) * sizeof(char*));
+			previous_cipher = mxrealloc(previous_cipher, (cur_thread + 1) * sizeof(char*));
+			previous_plaintext = mxrealloc(previous_plaintext, (cur_thread + 1) * sizeof(char*));
+		}
+	}
+
+
+	previous_ciphertext[cur_thread] = mxcalloc(1, mcrypt_get_block_size(algorithm));
+	previous_cipher[cur_thread] = mxmalloc(mcrypt_get_block_size(algorithm));
+	previous_plaintext[cur_thread] = mxcalloc(1, mcrypt_get_block_size(algorithm));
+
+	memmove(previous_ciphertext[cur_thread], IV, mcrypt_get_block_size(algorithm));
+	memmove(previous_plaintext[cur_thread], IV, mcrypt_get_block_size(algorithm));
+/* End cbc */
+
+	x = init_mcrypt(CBC, algorithm, cur_thread, key, lenofkey);
+
+	if (x < 0) {
+		end_mcrypt_cbc(cur_thread);
+		UNLOCK;
+		return -1;
+	} else {
+		UNLOCK;
+		return x;
+	}
+
+}
+
+
 int mcrypt_cbc(int td, void *plaintext, int len)
 {
 	word32 *fplain = plaintext;
@@ -709,7 +821,7 @@ int mdecrypt_cbc(int td, void *ciphertext, int len)
 			cipher[i] ^= previous_plaintext[td][i];
 		}
 
-		/* Copy the plaintext to prev_plaintext */
+		/* Copy the ciphertext to prev_cipher */
 		memmove(previous_plaintext[td], previous_cipher[td], blocksize);
 
 	}
@@ -724,10 +836,10 @@ int mdecrypt_cbc(int td, void *ciphertext, int len)
 int end_mcrypt_cfb(int td)
 {
 
-	free(s_register[td]);
-	free(enc_s_register[td]);
-	free(enc_sd_register[td]);
-	free(sd_register[td]);
+	mxfree(s_register[td], mcrypt_get_block_size(algorithm_given[td]));
+	mxfree(enc_s_register[td], mcrypt_get_block_size(algorithm_given[td]));
+	mxfree(enc_sd_register[td], mcrypt_get_block_size(algorithm_given[td]));
+	mxfree(sd_register[td], mcrypt_get_block_size(algorithm_given[td]));
 
 	return end_mcrypt(td);
 
@@ -738,36 +850,26 @@ int init_mcrypt_cfb(const int algorithm, void *key, int lenofkey, void *IV)
 {
 	int x;
 
-#ifdef HAVE_LIBPTHREAD
-	pthread_mutex_lock(&counter_cur_thread);
-#endif
-	cur_thread++;
+	LOCK;
 
+	if (cur_thread == MAX_THREADS) {
+		cur_thread = 0;
+		used_all_threads = 1;
+	}
+	while (check_pointer[cur_thread] == 1 && cur_thread < MAX_THREADS) {
+		cur_thread++;
+	}
 
-	if (cur_thread < MAX_THREADS && check_pointer[cur_thread] == 0) {
-
-/* for cfb */
-		if (used_all_threads == 0) {
+	if (cur_thread >= MAX_THREADS) {
+		UNLOCK;
+		return -1;
+	} else {
+                if (used_all_threads==0) {
 			s_register = mxrealloc(s_register, (cur_thread + 1) * sizeof(char *));
 			enc_s_register = mxrealloc(enc_s_register, (cur_thread + 1) * sizeof(char *));
 			enc_sd_register = mxrealloc(enc_sd_register, (cur_thread + 1) * sizeof(char *));
 			sd_register = mxrealloc(sd_register, (cur_thread + 1) * sizeof(char *));
 		}
-/* end cfb */
-	} else {
-		cur_thread = 0;
-		used_all_threads = 1;
-
-		while (check_pointer[cur_thread] == 1 && cur_thread <= MAX_THREADS) {
-			cur_thread++;
-		}
-
-		if (cur_thread >= MAX_THREADS) {
-#ifdef HAVE_LIBPTHREAD
-			pthread_mutex_unlock(&counter_cur_thread);
-#endif
-			return -1;
-		}
 	}
 
 
@@ -785,14 +887,10 @@ int init_mcrypt_cfb(const int algorithm, void *key, int lenofkey, void *IV)
 
 	if (x < 0) {
 		end_mcrypt_cfb(cur_thread);
-#ifdef HAVE_LIBPTHREAD
-		pthread_mutex_unlock(&counter_cur_thread);
-#endif
+		UNLOCK;
 		return -1;
 	} else {
-#ifdef HAVE_LIBPTHREAD
-		pthread_mutex_unlock(&counter_cur_thread);
-#endif
+		UNLOCK;
 		return x;
 	}
 }
@@ -854,10 +952,10 @@ int mdecrypt_cfb(int td, void *plaintext, int len)
 int end_mcrypt_ofb(int td)
 {
 
-	free(s_register[td]);
-	free(enc_s_register[td]);
-	free(enc_sd_register[td]);
-	free(sd_register[td]);
+	mxfree(s_register[td], mcrypt_get_block_size(algorithm_given[td]));
+	mxfree(enc_s_register[td], mcrypt_get_block_size(algorithm_given[td]));
+	mxfree(enc_sd_register[td], mcrypt_get_block_size(algorithm_given[td]));
+	mxfree(sd_register[td], mcrypt_get_block_size(algorithm_given[td]));
 
 	return end_mcrypt(td);
 
@@ -868,36 +966,26 @@ int init_mcrypt_ofb(const int algorithm, void *key, int lenofkey, void *IV)
 {
 	int x;
 
-#ifdef HAVE_LIBPTHREAD
-	pthread_mutex_lock(&counter_cur_thread);
-#endif
-	cur_thread++;
+	LOCK;
 
+	if (cur_thread == MAX_THREADS) {
+		cur_thread = 0;
+		used_all_threads = 1;
+	}
+	while (check_pointer[cur_thread] == 1 && cur_thread < MAX_THREADS) {
+		cur_thread++;
+	}
 
-	if (cur_thread < MAX_THREADS && check_pointer[cur_thread] == 0) {
-
-/* for ofb */
-		if (used_all_threads == 0) {
+	if (cur_thread >= MAX_THREADS) {
+		UNLOCK;
+		return -1;
+	} else {
+                if (used_all_threads==0) {
 			s_register = mxrealloc(s_register, (cur_thread + 1) * sizeof(char *));
 			enc_s_register = mxrealloc(enc_s_register, (cur_thread + 1) * sizeof(char *));
 			enc_sd_register = mxrealloc(enc_sd_register, (cur_thread + 1) * sizeof(char *));
 			sd_register = mxrealloc(sd_register, (cur_thread + 1) * sizeof(char *));
 		}
-/* end ofb */
-	} else {
-		cur_thread = 0;
-		used_all_threads = 1;
-
-		while (check_pointer[cur_thread] == 1 && cur_thread <= MAX_THREADS) {
-			cur_thread++;
-		}
-
-		if (cur_thread >= MAX_THREADS) {
-#ifdef HAVE_LIBPTHREAD
-			pthread_mutex_unlock(&counter_cur_thread);
-#endif
-			return -1;
-		}
 	}
 
 
@@ -915,14 +1003,10 @@ int init_mcrypt_ofb(const int algorithm, void *key, int lenofkey, void *IV)
 
 	if (x < 0) {
 		end_mcrypt_ofb(cur_thread);
-#ifdef HAVE_LIBPTHREAD
-		pthread_mutex_unlock(&counter_cur_thread);
-#endif
+		UNLOCK;
 		return -1;
 	} else {
-#ifdef HAVE_LIBPTHREAD
-		pthread_mutex_unlock(&counter_cur_thread);
-#endif
+		UNLOCK;
 		return x;
 	}
 }
@@ -985,10 +1069,10 @@ int mdecrypt_ofb(int td, void *plaintext, int len)
 int end_mcrypt_nofb(int td)
 {
 
-	free(s_register[td]);
-	free(enc_s_register[td]);
-	free(enc_sd_register[td]);
-	free(sd_register[td]);
+	mxfree(s_register[td], mcrypt_get_block_size(algorithm_given[td]));
+	mxfree(enc_s_register[td], mcrypt_get_block_size(algorithm_given[td]));
+	mxfree(enc_sd_register[td], mcrypt_get_block_size(algorithm_given[td]));
+	mxfree(sd_register[td], mcrypt_get_block_size(algorithm_given[td]));
 
 	return end_mcrypt(td);
 
@@ -999,36 +1083,26 @@ int init_mcrypt_nofb(const int algorithm, void *key, int lenofkey, void *IV)
 {
 	int x;
 
-#ifdef HAVE_LIBPTHREAD
-	pthread_mutex_lock(&counter_cur_thread);
-#endif
-	cur_thread++;
+	LOCK;
 
+	if (cur_thread == MAX_THREADS) {
+		cur_thread = 0;
+		used_all_threads = 1;
+	}
+	while (check_pointer[cur_thread] == 1 && cur_thread < MAX_THREADS) {
+		cur_thread++;
+	}
 
-	if (cur_thread < MAX_THREADS && check_pointer[cur_thread] == 0) {
-
-/* for nofb */
-		if (used_all_threads == 0) {
+	if (cur_thread >= MAX_THREADS) {
+		UNLOCK;
+		return -1;
+	} else {
+                if (used_all_threads==0) {
 			s_register = mxrealloc(s_register, (cur_thread + 1) * sizeof(char *));
 			enc_s_register = mxrealloc(enc_s_register, (cur_thread + 1) * sizeof(char *));
 			enc_sd_register = mxrealloc(enc_sd_register, (cur_thread + 1) * sizeof(char *));
 			sd_register = mxrealloc(sd_register, (cur_thread + 1) * sizeof(char *));
 		}
-/* end nofb */
-	} else {
-		cur_thread = 0;
-		used_all_threads = 1;
-
-		while (check_pointer[cur_thread] == 1 && cur_thread <= MAX_THREADS) {
-			cur_thread++;
-		}
-
-		if (cur_thread >= MAX_THREADS) {
-#ifdef HAVE_LIBPTHREAD
-			pthread_mutex_unlock(&counter_cur_thread);
-#endif
-			return -1;
-		}
 	}
 
 
@@ -1046,33 +1120,29 @@ int init_mcrypt_nofb(const int algorithm, void *key, int lenofkey, void *IV)
 
 	if (x < 0) {
 		end_mcrypt_nofb(cur_thread);
-#ifdef HAVE_LIBPTHREAD
-		pthread_mutex_unlock(&counter_cur_thread);
-#endif
+		UNLOCK;
 		return -1;
 	} else {
-#ifdef HAVE_LIBPTHREAD
-		pthread_mutex_unlock(&counter_cur_thread);
-#endif
+		UNLOCK;
 		return x;
 	}
 }
 
 int mcrypt_nofb(int td, void *plaintext, int len)
 {				/* plaintext is n*blocksize bytes (nbit ofb) */
-	word32 *plain, *fplain=plaintext;
+	word32 *plain, *fplain = plaintext;
 	int i, j, blocksize = mcrypt_get_block_size(algorithm_given[td]);
 
-	for (j = 0; j < len/blocksize; j++) {
-                plain = &fplain[ j * blocksize / sizeof(word32)];
+	for (j = 0; j < len / blocksize; j++) {
+		plain = &fplain[j * blocksize / sizeof(word32)];
 		memmove(enc_s_register[td], s_register[td], blocksize);
 		mcrypt(td, enc_s_register[td]);
 
-                for (i=0; i < blocksize/sizeof(word32); i++) {
-  			plain[i] ^= ((word32*) &enc_s_register[td][0+i*sizeof(word32)])[0];
-  		}
+		for (i = 0; i < blocksize / sizeof(word32); i++) {
+			plain[i] ^= ((word32 *) & enc_s_register[td][0 + i * sizeof(word32)])[0];
+		}
 /* Put the new register */
-                memmove( s_register[td], enc_s_register[td], blocksize);
+		memmove(s_register[td], enc_s_register[td], blocksize);
 	}
 	return 0;
 }
@@ -1080,20 +1150,20 @@ int mcrypt_nofb(int td, void *plaintext, int len)
 
 int mdecrypt_nofb(int td, void *plaintext, int len)
 {				/* plaintext is n bytes (nbit ofb) */
-	word32 *plain, *fplain=plaintext;
+	word32 *plain, *fplain = plaintext;
 	int i, j, blocksize = mcrypt_get_block_size(algorithm_given[td]);
 
-	for (j = 0; j < len/blocksize; j++) {
-		plain = &fplain[ j * blocksize / sizeof(word32)];
+	for (j = 0; j < len / blocksize; j++) {
+		plain = &fplain[j * blocksize / sizeof(word32)];
 		memmove(enc_sd_register[td], sd_register[td], blocksize);
 		mcrypt(td, enc_sd_register[td]);
 
 /* Shift the register */
-                memmove( sd_register[td], enc_sd_register[td], blocksize);
+		memmove(sd_register[td], enc_sd_register[td], blocksize);
 
-                for (i=0; i < blocksize/sizeof(word32); i++) {
-  			plain[i] ^= ((word32*) &enc_sd_register[td][0+i*sizeof(word32)])[0];
-  		}
+		for (i = 0; i < blocksize / sizeof(word32); i++) {
+			plain[i] ^= ((word32 *) & enc_sd_register[td][0 + i * sizeof(word32)])[0];
+		}
 	}
 
 	return 0;
@@ -1106,97 +1176,102 @@ int mdecrypt_nofb(int td, void *plaintext, int len)
 int mcrypt_generic_init(const int mode, const int algorithm, void *key, int lenofkey, void *IV)
 {
 
-  switch (mode) {
-    case CBC:
-      return init_mcrypt_cbc( algorithm, key, lenofkey);
-      break;
-    case ECB:
-      return init_mcrypt_ecb( algorithm, key, lenofkey);
-      break;
-    case CFB:
-      return init_mcrypt_cfb( algorithm, key, lenofkey, IV);
-      break;
-    case OFB:
-      return init_mcrypt_ofb( algorithm, key, lenofkey, IV);
-      break;
-    case nOFB:
-      return init_mcrypt_nofb( algorithm, key, lenofkey, IV);
-      break;
-    default: 
-      return -1;
-  }
+	switch (mode) {
+	case CBC:
+		if (IV == NULL) {
+			return init_mcrypt_cbc(algorithm, key, lenofkey);
+		} else {
+			return init_mcrypt_cbc_iv(algorithm, key, lenofkey, IV);
+		}
+		break;
+	case ECB:
+		return init_mcrypt_ecb(algorithm, key, lenofkey);
+		break;
+	case CFB:
+		return init_mcrypt_cfb(algorithm, key, lenofkey, IV);
+		break;
+	case OFB:
+		return init_mcrypt_ofb(algorithm, key, lenofkey, IV);
+		break;
+	case nOFB:
+		return init_mcrypt_nofb(algorithm, key, lenofkey, IV);
+		break;
+	default:
+		return -1;
+	}
 }
 
 int mcrypt_generic(int td, void *plaintext, int len)
 {
 
- switch (mode_given[td]) {
-    case CBC:
-      return mcrypt_cbc(td, plaintext, len);
-      break;
-    case ECB:
-      return mcrypt_ecb(td, plaintext, len);
-      break;
-    case CFB:
-      return mcrypt_cfb(td, plaintext, len);
-      break;
-    case OFB:
-      return mcrypt_ofb(td, plaintext, len);
-      break;
-    case nOFB:
-      return mcrypt_nofb(td, plaintext, len);
-      break;
-    default: 
-      return -1;
-  }
+	switch (mode_given[td]) {
+	case CBC:
+		return mcrypt_cbc(td, plaintext, len);
+		break;
+	case ECB:
+		return mcrypt_ecb(td, plaintext, len);
+		break;
+	case CFB:
+		return mcrypt_cfb(td, plaintext, len);
+		break;
+	case OFB:
+		return mcrypt_ofb(td, plaintext, len);
+		break;
+	case nOFB:
+		return mcrypt_nofb(td, plaintext, len);
+		break;
+	default:
+		return -1;
+	}
 
 }
 
-int mdecrypt_generic (int td, void *ciphertext, int len)
+int mdecrypt_generic(int td, void *ciphertext, int len)
 {
 
- switch (mode_given[td]) {
-    case CBC:
-      return mdecrypt_cbc(td, ciphertext, len);
-      break;
-    case ECB:
-      return mdecrypt_ecb(td, ciphertext, len);
-      break;
-    case CFB:
-      return mdecrypt_cfb(td, ciphertext, len);
-      break;
-    case OFB:
-      return mdecrypt_ofb(td, ciphertext, len);
-      break;
-    case nOFB:
-      return mdecrypt_nofb(td, ciphertext, len);
-      break;
-    default: 
-      return -1;
-  }
+	switch (mode_given[td]) {
+	case CBC:
+		return mdecrypt_cbc(td, ciphertext, len);
+		break;
+	case ECB:
+		return mdecrypt_ecb(td, ciphertext, len);
+		break;
+	case CFB:
+		return mdecrypt_cfb(td, ciphertext, len);
+		break;
+	case OFB:
+		return mdecrypt_ofb(td, ciphertext, len);
+		break;
+	case nOFB:
+		return mdecrypt_nofb(td, ciphertext, len);
+		break;
+	default:
+		return -1;
+	}
 
 }
 
 int mcrypt_generic_end(const int td)
 {
 
-  switch (mode_given[td]) {
-    case CBC:
-      return end_mcrypt_cbc(td);
-      break;
-    case ECB:
-      return end_mcrypt_ecb(td);
-      break;
-    case CFB:
-      return end_mcrypt_cfb(td);
-      break;
-    case OFB:
-      return end_mcrypt_ofb(td);
-      break;
-    case nOFB:
-      return end_mcrypt_nofb(td);
-      break;
-    default: 
-      return -1;
-  }
+	switch (mode_given[td]) {
+	case CBC:
+		return end_mcrypt_cbc(td);
+		break;
+	case ECB:
+		return end_mcrypt_ecb(td);
+		break;
+	case CFB:
+		return end_mcrypt_cfb(td);
+		break;
+	case OFB:
+		return end_mcrypt_ofb(td);
+		break;
+	case nOFB:
+		return end_mcrypt_nofb(td);
+		break;
+	default:
+		return -1;
+	}
 }
+
