@@ -27,7 +27,6 @@
 #include <gost.h>
 #include <des.h>
 #include <safer.h>
-#include <cast.h>
 #include <tean.h>
 #include <aes.h>
 #include <xmemory.h>
@@ -35,6 +34,8 @@
 #include <rc6.h>
 #include <idea.h>
 #include <sha1.h>
+#include <md5.h>
+#include <loki97.h>
 
 int mcrypt_get_block_size(int algorithm)
 {
@@ -42,7 +43,10 @@ int mcrypt_get_block_size(int algorithm)
 
 	case DES:
 	case TripleDES:
-	case BLOWFISH:
+	case BLOWFISH448:
+	case BLOWFISH256:
+	case BLOWFISH192:
+	case BLOWFISH128:
 	case GOST:
 	case SAFER64:
 	case SAFER128:
@@ -62,6 +66,9 @@ int mcrypt_get_block_size(int algorithm)
 		break;
 #endif
 
+	case LOKI97:
+	case SAFERPLUS:
+	case CAST256:
 	case TWOFISH128:
 	case TWOFISH192:
 	case TWOFISH256:
@@ -92,17 +99,32 @@ int mcrypt_get_key_size(int algorithm)
 		return 24;
 		break;
 
-	case BLOWFISH:
+	case BLOWFISH448:
 		return 56;
 		break;
 
+	case LOKI97:
+	case BLOWFISH256:
+	case SAFERPLUS:
+		return 32;
+		break;
+		
+	case BLOWFISH192:
+		return 24;
+		break;
+		
+	case BLOWFISH128:
+		return 16;
+		break;
+
 	case RC2:
 		return 128;
 		break;
 
 #ifdef NON_FREE
 	case RC6:
-		return RC6_BKEY;	/* 32 */
+//		return RC6_BKEY;	/* 32 */
+		return 32;
 		break;
 #endif
 
@@ -111,6 +133,7 @@ int mcrypt_get_key_size(int algorithm)
 		break;
 
 	case GOST:
+	case CAST256:
 	case TWOFISH256:
 		return 32;
 		break;
@@ -178,11 +201,30 @@ char *get_algorithms_name(int algorithm)
 		return x;
 		break;
 
-	case BLOWFISH:
-		strcpy(x, "Blowfish");
+	case BLOWFISH448:
+		strcpy(x, "Blowfish-448");
 		return x;
 		break;
 
+	case BLOWFISH256:
+		strcpy(x, "Blowfish-256");
+		return x;
+		break;
+
+	case BLOWFISH192:
+		strcpy(x, "Blowfish-192");
+		return x;
+		break;
+
+	case BLOWFISH128:
+		strcpy(x, "Blowfish-128");
+		return x;
+		break;
+
+	case CAST256:
+		strcpy(x, "CAST-256");
+		return x;
+		break;	
 	case TWOFISH128:
 		strcpy(x, "Twofish-128");
 		return x;
@@ -201,6 +243,16 @@ char *get_algorithms_name(int algorithm)
 		return x;
 		break;
 
+	case LOKI97:
+		strcpy(x, "LOKI97");
+		return x;
+		break;
+		
+	case SAFERPLUS:
+		strcpy(x, "SAFER+");
+		return x;
+		break;
+
 	case SAFER64:
 		strcpy(x, "SAFER-SK64");
 		return x;
@@ -236,7 +288,11 @@ int is_ok_algorithm(int algorithm)
 #endif
 	case TripleDES:
 	case ThreeWAY:
-	case BLOWFISH:
+	case BLOWFISH448:
+	case BLOWFISH256:	
+	case BLOWFISH192:
+	case BLOWFISH128:
+	case CAST256:
 	case TWOFISH128:
 	case TWOFISH192:
 	case TWOFISH256:
@@ -245,6 +301,8 @@ int is_ok_algorithm(int algorithm)
 	case SAFER128:
 	case CAST128:
 	case TEAN:
+	case SAFERPLUS:
+	case LOKI97:
 		return 0;
 		break;
 
@@ -276,40 +334,6 @@ int is_ok_mode(int mode)
 }
 
 
-void gen_key(void *keyword, void *salt, int salt_size, int key_size,
-	     unsigned char *password, int plen)
-{
-	unsigned char key[MAX_KEY_LEN];
-	SHA1_CTX ctx;
-	unsigned char *digest;
-	int size = key_size;
-	unsigned char *cp = key, *cp0 = key;
-	int salt_z = 0;		/* flag to be used when freeing salt[] */
-
-	if (salt == NULL) {
-		salt_z = 1;
-	}
-	while (1) {
-		SHA1Init(&ctx);
-		if (salt_z == 0)
-			SHA1Update(&ctx, salt, salt_size);
-		SHA1Update(&ctx, password, plen);
-		if (cp - cp0 > 0)
-			SHA1Update(&ctx, cp0, cp - cp0);
-		digest = SHA1Final(&ctx);
-		if (size > 20) {
-			memmove(cp, digest, 20);
-			free(digest);
-			size -= 20;
-			cp += 20;
-		} else {
-			memmove(cp, digest, size);
-			free(digest);
-			break;
-		}
-	}
-	memmove(keyword, key, key_size);
-}
 
 
 char *get_modes_name(int mode)
@@ -349,3 +373,74 @@ char *get_modes_name(int mode)
 }
 
 
+/* Key generation using sha1 */
+void gen_key_sha1(void *keyword, void *salt, int salt_size, int key_size,
+	     unsigned char *password, int plen)
+{
+	unsigned char key[MAX_KEY_LEN];
+	SHA1_CTX ctx;
+	unsigned char *digest;
+	int size = key_size;
+	unsigned char *cp = key, *cp0 = key;
+	int salt_z = 0;		/* flag to be used when freeing salt[] */
+
+	if (salt == NULL) {
+		salt_z = 1;
+	}
+	while (1) {
+		SHA1Init(&ctx);
+		if (salt_z == 0)
+			SHA1Update(&ctx, salt, salt_size);
+		SHA1Update(&ctx, password, plen);
+		if (cp - cp0 > 0)
+			SHA1Update(&ctx, cp0, cp - cp0);
+		digest = SHA1Final(&ctx);
+		if (size > 20) {
+			memmove(cp, digest, 20);
+			mxfree(digest, 20);
+			size -= 20;
+			cp += 20;
+		} else {
+			memmove(cp, digest, size);
+			mxfree(digest, size);
+			break;
+		}
+	}
+	memmove(keyword, key, key_size);
+}
+
+
+void gen_key_md5(void *keyword, void *salt, int salt_size, int key_size,
+	     unsigned char *password, int plen)
+{
+	unsigned char key[MAX_KEY_LEN];
+	MD5_CTX ctx;
+	unsigned char *digest;
+	int size = key_size;
+	unsigned char *cp = key, *cp0 = key;
+	int salt_z = 0;
+
+	if (salt == NULL) {
+		salt_z = 1;
+	}
+	while (1) {
+		MD5Init(&ctx);
+		if (salt_z == 0)
+			MD5Update(&ctx, salt, salt_size);
+		MD5Update(&ctx, password, plen);
+		if (cp - cp0 > 0)
+			MD5Update(&ctx, cp0, cp - cp0);
+		digest = MD5Final(&ctx);
+		if (size > 16) {
+			memmove(cp, digest, 16);
+			free(digest);
+			size -= 16;
+			cp += 16;
+		} else {
+			memmove(cp, digest, size);
+			free(digest);
+			break;
+		}
+	}
+	memmove(keyword, key, key_size);
+}
