@@ -74,7 +74,7 @@ key_buf=mxcalloc(1, 24);
   memmove(keyword2,&key_buf[8],8); 
   memmove(keyword3,&key_buf[16],8); 
 
-  free(key_buf);
+  mxfree(key_buf, 24);
 
   return 0;
 
