@@ -139,7 +139,7 @@ CONST		char *modeString		=MOD_STRING;
 #define		CHECK_TABLE		0		/* nonzero --> compare against "slow" table */
 #define		VALIDATE_PARMS	0		/* disable for full speed */
 
-#include	<debug.h>				/* debug display macros */
+#include	"debug.h"				/* debug display macros */
 
 /* end of debug macros */
 
@@ -740,7 +740,7 @@ else
 * Notes:	This parses the key bits from keyMaterial.  Zeroes out unused key bits
 *
 -****************************************************************************/
-int makeKey(keyInstance *key, BYTE direction, int keyLen,CONST char *keyMaterial)
+int _mcrypt_makeKey(keyInstance *key, BYTE direction, int keyLen,CONST char *keyMaterial)
 	{
 #if VALIDATE_PARMS				/* first, sanity check on parameters */
 	if (key == NULL)			
@@ -787,7 +787,7 @@ int makeKey(keyInstance *key, BYTE direction, int keyLen,CONST char *keyMaterial
 *					else error code (e.g., BAD_CIPHER_MODE)
 *
 -****************************************************************************/
-int cipherInit(cipherInstance *cipher, BYTE mode,CONST char *IV)
+int _mcrypt_cipherInit(cipherInstance *cipher, BYTE mode,CONST char *IV)
 	{
 	int i;
 #if VALIDATE_PARMS				/* first, sanity check on parameters */
@@ -839,7 +839,7 @@ int cipherInit(cipherInstance *cipher, BYTE mode,CONST char *IV)
 *		 sizes can be supported.
 *
 -****************************************************************************/
-int blockEncrypt(cipherInstance *cipher, keyInstance *key,CONST BYTE *input,
+int _mcrypt_blockEncrypt(cipherInstance *cipher, keyInstance *key,CONST BYTE *input,
 				int inputLen, BYTE *outBuffer)
 	{
 	int   i,n;						/* loop counters */
@@ -878,7 +878,7 @@ int blockEncrypt(cipherInstance *cipher, keyInstance *key,CONST BYTE *input,
 		cipher->mode = MODE_ECB;	/* do encryption in ECB */
 		for (n=0;n<inputLen;n++)
 			{
-			blockEncrypt(cipher,key,cipher->IV,BLOCK_SIZE,(BYTE *)x);
+			_mcrypt_blockEncrypt(cipher,key,cipher->IV,BLOCK_SIZE,(BYTE *)x);
 			bit	  = 0x80 >> (n & 7);/* which bit position in byte */
 			ctBit = (input[n/8] & bit) ^ ((((BYTE *) x)[0] & 0x80) >> (n&7));
 			outBuffer[n/8] = (outBuffer[n/8] & ~ bit) | ctBit;
@@ -1010,7 +1010,7 @@ int blockEncrypt(cipherInstance *cipher, keyInstance *key,CONST BYTE *input,
 *		 sizes can be supported.
 *
 -****************************************************************************/
-int blockDecrypt(cipherInstance *cipher, keyInstance *key,CONST BYTE *input,
+int _mcrypt_blockDecrypt(cipherInstance *cipher, keyInstance *key,CONST BYTE *input,
 				int inputLen, BYTE *outBuffer)
 	{
 	int   i,n;						/* loop counters */
@@ -1049,7 +1049,7 @@ int blockDecrypt(cipherInstance *cipher, keyInstance *key,CONST BYTE *input,
 		cipher->mode = MODE_ECB;	/* do encryption in ECB */
 		for (n=0;n<inputLen;n++)
 			{
-			blockEncrypt(cipher,key,cipher->IV,BLOCK_SIZE,(BYTE *)x);
+			_mcrypt_blockEncrypt(cipher,key,cipher->IV,BLOCK_SIZE,(BYTE *)x);
 			bit	  = 0x80 >> (n & 7);
 			ctBit = input[n/8] & bit;
 			outBuffer[n/8] = (outBuffer[n/8] & ~ bit) |
