@@ -19,6 +19,21 @@
 # define LIBDEFS_H
 # include <libdefs.h>
 #endif
+#include <bzero.h>
+
+#ifdef HAVE_SYS_MMAN_H
+# include <sys/mman.h>
+#endif
+
+#ifdef HAVE_MLOCK
+void LOCKMEM( void* x, size_t size) {
+ mlock(x,size);
+}
+# define UNLOCKMEM(x,y) munlock(x,y)
+#else
+# define LOCKMEM(x,y)
+# define UNLOCKMEM(x,y)
+#endif
 
 #include <bzero.h>
 #include <xmemory.h>
@@ -29,13 +44,16 @@ void *mxmalloc(size_t size) {
 char *x;
 
  x=malloc(size);
+ 
  if (x!=NULL) {
+  LOCKMEM(x,size);
   return x;
  }
  else {
   sleep(1);
   x=malloc(size);
   if (x!=NULL) {
+   LOCKMEM(x,size);
    return x;
   }
   else {
@@ -52,13 +70,15 @@ char *x;
 
  x=calloc(nmemb, size);
  if (x!=NULL) {
-  return x;
+    return x;
+    LOCKMEM(x,size);
  }
  else {
   sleep(1);
   x=calloc(nmemb, size);
   if (x!=NULL) {
-   return x;
+    LOCKMEM(x,size);
+    return x;
   }
   else {
    fprintf(stderr, ("Cannot allocate memory\n"));
@@ -75,13 +95,15 @@ char *x;
 
  x=realloc(ptr, size);
  if (x!=NULL) {
-  return x;
+    LOCKMEM(x,size);
+    return x;
  }
  else {
   sleep(1);
   x=realloc(ptr, size);
   if (x!=NULL) {
-   return x;
+    LOCKMEM(x,size);
+    return x;
   }
   else {
    fprintf(stderr, ("Cannot allocate memory\n"));
@@ -89,8 +111,16 @@ char *x;
   }
  }
 /* Not really needed */
+/* Just for some compilers which complain */
  return x;
 
 }
 
 
+void mxfree(void *ptr, size_t size) {
+
+  Bzero( ptr, size);
+  UNLOCKMEM(ptr ,size);
+  free(ptr);
+          
+}
